from .util import *
from .mploter import Plotor
import matplotlib.pyplot as plt
import time
import statsmodels.api as sm

def eval_factor(x, y, show=True, save_dir='%s/MultiPlot/'%(home_path()), fig_name = 'default', time=[], date=[]):
  Assert(len(x) == len(y), 'Evaling a length not align factor %d %d'%(len(x), len(y)))
  if len(date) != 0: Assert(len(date) == len(x), 'Passing Date, but len(date) != len(x) %d %d'%(len(date), len(x)))
  df = x.to_frame()
  df[y.name] = y
  if len(time) > 0: df = df.between_time(time[0], time[1], inclusive='left')
  df['group'] = pd.cut(df.index, 10, labels=False)
  gp = df.groupby('group')
  ts_ic  = gp.apply(lambda a: a[x.name].corr(a[y.name]))
  ts_psh = gp.apply(lambda a: (a[y.name]).sum() / a[x.name].abs().sum()) # ensure y is netual
  df['qcut'] = pd.qcut(df[x.name], 1000, duplicates='drop', labels=False)
  gp_qcut = df.groupby('qcut')
  sort_psh = gp_qcut.apply(lambda a: (a[y.name]).mean())# / a[x.name].abs().sum())
  sort_ic  = gp_qcut.apply(lambda a: a[x.name].corr(a[y.name]))
  #print(sort_psh)
  #print(sort_ic)
  #print(ts_psh)
  #print(ts_ic)
  #m = {'ic': {'ts@bar':ts_ic, 'sort@bar':sort_ic}, 'psh':{'ts@bar':ts_psh, 'sort@bar':sort_psh}, 'effsize':eff_plot, 'ts_x':x}
  m = {'sort': {'psh@bar':sort_psh}, 'time_series':{'ic=%.3f@bar'%(df[x.name].corr(df[y.name])):ts_ic, 'psh@twin':ts_psh}, 'effsize':{x.name : (x*y).cumsum()}, 'factor_hist':{'a@hist':x}}
  Plotor.MultiPlot3(m, show=show, save_dir=save_dir, fig_name = fig_name)

@timer
def see_predlength(x, ylist, date = None):
  #Assert(len(x) == len(y), 'Evaling a length not align factor %d %d'%(len(x), len(y)))
  #if date != None: Assert(len(date) == len(x), 'Passing Date, but len(date) != len(x) %d %d'%(len(date), len(x)))
  m, m2 = {}, {}
  for y in ylist: m[y] = x.corr(ylist[y])
  for y in ylist: m2[y] = x.corr(ylist[y], method='spearman')
  plt.plot(m.values(), label='pearson')
  plt.xticks(range(len(m)), list(m.keys()))
  plt.twinx()
  plt.plot(m2.values(), label='spearman')
  plt.xticks(range(len(m2)), list(m2.keys()))
  plt.legend()
  plt.show()

def ols_check(x, y): 
  Assert(len(x) == len(y), "OLS X Y length not equal")
  df = pd.DataFrame()
  df['x'] = list(x)
  df['y'] = list(y)
  df = df.dropna()
  if len(df) / (len(x)*1.) < 0.5: RedPrint('drop na more than 50%, length from %d -> %d'%(len(x), len(df)))
  x = df[['x']]
  y = df['y']
  x = sm.add_constant(x)
  print(sm.OLS(y, x).fit().summary())

def find_corr(df, greater=True, thr=0.6):
  m = []
  for i, (col, row) in enumerate(df.iterrows()):
    for j, (index, val) in enumerate(zip(row.index, row)):
      if i >= j: continue
      if greater and abs(val) > thr:m.append([col, index, val])
      if not greater and abs(val) < thr: m.append([col, index, col])
  return pd.DataFrame(m)

if __name__ == '__main__':
  from ..hft_struct.Reader import Reader
  r = Reader()
  df = r.load({'IC1@cffexl2'}, '20210101', '$TODAY')
  #eval_factor(df['vol_'], df['ap0_'])
  ols_check(df['vol_'], df['ap0_'])



import time
import pandas as pd
import struct

DEPTH = 5
class MarketSnapshot:
  def __init__(self):
    self.tk_= "none"
    self.ts_ = 1000000000
    self.uts_ = 1000
    for i in range(DEPTH): exec('self.bp%d_=0.'%(i))
    for i in range(DEPTH): exec('self.ap%d_=0.'%(i))
    for i in range(DEPTH): exec('self.bs%d_=0'%(i))
    for i in range(DEPTH): exec('self.as%d_=0'%(i))
    self.lp_ = 0.
    self.tvr_ = 0.
    self.ls_ = 0
    self.vol_ = 0
    self.oi_ = 0
  
  @property
  def fmt(self): return '8s16s2Q10d10idd3i4s'

  def pack(self):
    return struct.pack(self.fmt, bytes(self.tk_, 'utf-8'), bytes(self.tk_, 'utf-8'), self.ts_, self.uts_, self.bp0_, self.bp1_, self.bp2_, self.bp3_, self.bp4_, self.ap0_, self.ap1_, self.ap2_, self.ap3_, self.ap4_, self.bs0_, self.bs1_, self.bs2_, self.bs3_, self.bs4_, self.as0_, self.as1_, self.as2_, self.as3_, self.as4_, self.lp_, self.tvr_, self.ls_, self.vol_, self.oi_, b'')

  def Check(self, ts_check=False):
    if self.price_check:
      if self.bs0_ < 0.1 or self.as0_ < 0.1 or self.lp_ < 0.1 or self.bp0_ < 0.1 or self.ap0_ < 0.1 or len(self.tk_) > 20: return False
    if self.tk_ == 'tk': return False
    if self.tk_ == 'CODE': return False
    return True

  def __str__(self):
    split_char = ' '
    show_content = ""
    show_content += str(self.ts_) + '.' + str(self.uts_)
    show_content += split_char
    show_content += "SNAPSHOT"
    show_content += split_char
    show_content += self.tk_
    show_content += split_char
    show_content += '|'
    show_content += split_char
    for i in range(DEPTH): show_content += str(eval('self.bp%d_'%(i))) + split_char + str(eval('self.ap%d_'%(i))) + ' | ' + str(eval('self.bs%d_'%(i))) + ' x ' + str(eval('self.as%d_'%(i))) + ' |' + split_char
    show_content += str(self.lp_)
    show_content += split_char
    show_content += str(self.ls_)
    show_content += split_char
    show_content += str(self.vol_)
    show_content += split_char
    show_content += 'M'
    show_content += split_char
    show_content += str(self.tvr_)
    show_content += split_char
    show_content += str(self.oi_)
    show_content += split_char
    return show_content

if __name__ == '__main__':
  shot = MarketSnapshot()
  shot.tk = "HAHA"
  m = {}
  for i, j in zip(shot.cols, shot.to_csv().split(',')): m[i] = j
  print(pd.Series(m))



from .market_snapshot import *
from .book_ticker import *
from .order import *
from .exchange_info import *
from collections.abc import Iterable
import sys

def Show(s, f=None):
  if f: f.write(s.__str__()+'\n')
  else: print(s)

def ShowCsv(s, f=None):
  if f: f.write(s.to_csv() + '\n')
  else: print(s.to_csv() + '\n')

def to_csv(s, split_c=','): return split_c.join([str(val) for val in s.__dict__.values()])

def to_series(s): return pd.Series({key:val for key, val in s.__dict__.items()})

def unpack(s, text):
  #print(len(struct.unpack(s.fmt, text)))
  #print(len(s.__dict__.keys()))
  for key, val in zip(s.__dict__.keys(), struct.unpack(s.fmt, text)):
    if isinstance(val, bytes): exec('s.%s=val.decode("utf-8")'%(key))
    else: exec('s.%s=val'%(key))
  return s

def get_size(s): return struct.calcsize(s.fmt)

if __name__ == '__main__':
  shot = MarketSnapshot()
  with open('/QData/market_data/futures/tick/ctp_zhaoshang/20210714/NI1.20210714.bin', 'rb') as f:
    s = f.read()
    unpack(shot, s[100*struct.calcsize(shot.fmt):101*struct.calcsize(shot.fmt)])
    sys.exit(1)



import numpy as np
import datetime as dt

class Order:
  def __init__(self):
    self.shot_time=1000000000.0;
    self.send_time=1000000000.0;
    self.ticker=''
    self.price=-1.0
    self.size=1
    self.traded_size=1
    self.side=-1
    self.order_ref=''
    self.action = -1
    self.status = -1
    self.offset=-1
    self.target_price = -1
    self.shot_latency = 0.0
    self.exchange = ''
    self.tbd=''
    action_list = ['Uninited', 'NewOrder', 'ModOrder', 'CancelOrder', 'QueryPos', 'PlainText']
    status_list = ['Uninited', 'SubmitNew', 'New', 'Rejected', 'Modifying', 'Cancelling', 'Cancelled', 'CancelRej', 'Pfilled', 'Filled', 'Sleep']
    self.action_map = {i:action_list[i] for i in range(len(action_list))}
    self.action_map[-1] = 'Unknown'
    self.status_map = {i:status_list[i] for i in range(len(status_list))}
    self.status_map[-1] = 'Unknown'

  @property
  def fmt(self): return "4Q32sd3i32s3i2d96s32s"

  def __str__(self):
    split_c = ' '
    show_str = ''
    show_str += repr(self.shot_time)
    show_str += split_c   
    show_str += repr(self.send_time)
    show_str += split_c + 'Order' + split_c
    show_str += str(self.ticker)
    show_str += split_c   
    show_str += str(self.price)
    show_str += '@'
    show_str += str(self.size)
    show_str += split_c   
    show_str += str(self.traded_size)
    show_str += split_c
    show_str += "BUY" if self.side == 1 else "SELL"
    show_str += split_c
    show_str += str(self.action_map[self.action])
    show_str += split_c   
    show_str += str(self.status_map[self.status])
    show_str += split_c
    show_str += str(self.shot_latency)
    show_str += split_c
    show_str += self.order_ref
    show_str += split_c   
    show_str += self.tbd
    show_str += split_c   
    show_str += self.exchange
    if self.shot_time < 1e10 and self.shot_time > 1e9:
      show_str += split_c   
      show_str += str(dt.datetime.fromtimestamp(int(self.shot_time)).time())
    return show_str

  def Check(self): return self.action != 4

if __name__=='__main__':
  o = Order()
  with open('/root/order.log') as f:
    for l in f:
      o.construct(l)
      o.Show()



import pandas as pd
import numpy as np
import psutil
from tqdm import tqdm
import glob
from numpy import random
import os
import sys
import datetime as dt
from functools import reduce
import multiprocessing
import traceback
from queue import Queue
from collections.abc import Iterable
from collections import defaultdict
import time
import warnings
from tabulate import tabulate
from termcolor import colored
warnings.filterwarnings('ignore')

def timer(func):
  def wrapper(*args, **kwargs):
    s = time.time()
    r = func(*args, **kwargs)
    RedPrint('['+func.__name__+']', 'cost', time.time()-s, color='green')
    return r
  return wrapper

def header(func):
  def wrapper(*args, **kwargs):
    h(func.__name__)
    r = func(*args, **kwargs)
    t(func.__name__)
    return r
  return wrapper

def GetCon(ticker):
  for i, c in enumerate(ticker):
    if c.isdigit(): return ticker[:i]
  return ticker

def loop(func, params):
  pbar = tqdm(total=len(params), desc='Looping', colour="cyan")
  for param in params:
    pbar.set_description('Processing %s'%(str(param)))
    if isinstance(param, list): func(*param)
    else: func(param)
    pbar.update(1)

def current_time():
  sec = tme.mktime(dt.datetime.now().timetuple())
  usec = dt.datetime.now().microsecond
  return sec + usec / 1e7

def get_date(delta = 0):
  return (dt.date.today() + dt.timedelta(days=delta)).strftime('%Y%m%d')

def dateRange(beginDate, endDate, split_c='-'):
  split_c='-' if '-' in beginDate else ''
  dates = []
  d = dt.datetime.strptime(beginDate, '%Y' + split_c + '%m' + split_c + '%d')
  date = beginDate[:]
  while date <= endDate:
    dates.append(date)
    d = d + dt.timedelta(1)
    date = d.strftime('%Y' + split_c + '%m' + split_c + '%d')
  return dates

def Throw(func, args=(), dicts={}):
  multiprocessing.Process(target=func, args=args, kwargs=dicts).start()

@timer
def MPRun(func, args, dicts={}, n_pool= max(multiprocessing.cpu_count()-2, 1)):
  RedPrint('submit a multiprocess task with cpu', n_pool, color='purple')
  n_pool = multiprocessing.cpu_count()-n_pool-1 if n_pool < 0 else n_pool
  q = Queue()
  if not isinstance(args, list) or not isinstance(dicts, dict): RedPrint('args should be list, dicts should be dict', type(args), type(dicts)); sys.exit(1)
  pool = multiprocessing.Pool(processes=n_pool)
  res = []
  pbar = tqdm(total=len(args), desc=ColorString('MPRun %s'%(func.__name__), color='blue'), colour="cyan")
  for i, arg in enumerate(args):
    if not isinstance(args[0], list): q.put(pool.apply_async(func, tuple([arg]), dicts))
    else: q.put(pool.apply_async(func, tuple(arg), dicts))
  count = 0
  r = []
  while count < len(args):
    r.append(q.get().get())
    count += 1
    pbar.update(1)
  pbar.close()
  pool.close()
  pool.join()
  return r

def home_path():
  return os.environ['HOME']+'/'

color_list = ['black', 'red', 'green', 'yellow', 'blue', 'purple', 'cran', 'white']
fore_color = {c: i + 30 for i, c in enumerate(color_list)}
back_color = {c: i + 40 for i, c in enumerate(color_list)}
#show_list = ['default', 'highlight', 'underline', 'shinning', 'reverse', 'invisiable']
#show_way = {sl : i for i, sl in enumerate(show_list)}

def pretty_show(d, rd=-1):
  print(tabulate(d.round(rd) if rd > 0 else d, tablefmt='pretty', numalign='left', headers=d.columns))
  #RedPrint(tabulate(d.round(round_digit), tablefmt='pretty', numalign='left', headers=d.columns), color='blue')

def show_with_color(df, thr = 0., rd = -1):
  for col in df.columns: df[col] = [colored(str(x), None, 'on_red') if x < thr else str(x) for x in df[col]]
  pretty_show(df, rd)

def ColorString(*s, color='red'):
  fore, back = 30, 40
  if color in fore_color: fore = fore_color[color]
  if color in back_color: back = back_color[color]
  return '\033[%dm%s\033[0m'%(fore, ' '.join(map(lambda x: str(x), s)))

def RedPrint(*s, color = 'red', show='highlight'):
  print(ColorString(*s, color=color))

def h(*s, color='red', num = 13, chr ='#'):
  s = map(lambda x: str(x), s)
  RedPrint(chr*num, 'HEAD['+' '.join(s)+']', chr*num, color=color)

def t(*s, color='red', num = 13, chr ='#'):
  s = map(lambda x: str(x), s)
  RedPrint(chr*num, 'TAIL['+' '.join(s)+']', chr*num, color=color)

def p(*s, color='green'):
  te = traceback.extract_stack()
  c = [i[2] for i in traceback.extract_stack()]
  call_seq = c[-c[::-1].index('<module>'):]
  prefix = '[' + '->'.join(call_seq) + ']'
  s = map(lambda x: str(x), s)
  RedPrint(prefix, ' '.join(s), color=color)

def Assert(cond, *s, color='purple'):
  te = traceback.extract_stack()
  c = [i[2] for i in traceback.extract_stack()]
  call_seq = c[-c[::-1].index('<module>'):]
  prefix = '[' + '->'.join(call_seq) + ']'
  cs = ColorString(prefix+ 'Assert failed ' + ' '.join(s), color=color)
  assert cond, cs

def checkargtypes(args, func):
  for argname, annotation in func.__annotations__.items():
    if argname == "return": continue
    if type(args[argname]) is not type(annotation):
      print("the type of arg '{}' may be inappropriate.".format(argname))
      return False
  return True

def get_all_function(cls):
  funcs = []
  for i in cls.__dir__():
    if i.startswith('__'): continue
    if hasattr(eval('cls.%s'%(i)), '__call__'): funcs.append(eval('cls.%s'%(i)))
  return funcs

def file_dur(path):
  return (pd.to_datetime(os.path.getatime(path), unit='s').date()-dt.datetime.today().date()) / dt.timedelta(1)

def file_date(path):
  return str(pd.to_datetime(os.path.getatime(path), unit='s').date())

def get_process_memory():
  RedPrint(u'当前进程的内存使用：%.4f GB' % (psutil.Process(os.getpid()).memory_info().rss / 1024 / 1024 / 1024), color='purple')

def GetExpire(df):
  year = (df['date'] / 10000).apply(int)
  y_tag = year % 10
  ticker_y_tag = df['ticker'].str.slice(-3, -2).apply(int)
  ticker_m_tag = df['ticker'].str.slice(-2,).apply(int)
  y_dif = (ticker_y_tag - y_tag + 10) % 10
  a = y_dif[(y_dif > 2) & (y_dif < 0)]
  Assert(len(a) == 0, 'expire date illegal')
  rdate = (year * 10000 + ticker_m_tag * 100 + 28) + (y_dif * 10000)
  #Assert((rdate > df['date']).mean()==1)
  return rdate
 
def reload(lib):
  s = 'import importlib; import %s; importlib.reload(%s); from %s import *;'%(lib, lib, lib)
  return s

def get_default(m, key, default):
  if key in m:
    value = m[key]
    Assert(type(value) == type(default), '%s in params, but type is wrong %s, should be %s' %(key, type(m[key]), type(default)))
    return value
  return default

if __name__ == '__main__':
  #MPRun(A.print, [[a,1],[a,2],[a,3]])
  #Assert(1 == 2)
  print(GetExpire(20191111, 'M102'))



from .util import *
import matplotlib
import matplotlib.pyplot as plt
import math
import datetime
import matplotlib.dates as mdate
from collections import defaultdict
from statsmodels.graphics.tsaplots import plot_acf, plot_pacf

class Plotor:
  def __init__(self, one_width=5, area=80):
    pass

  @staticmethod
  def plot_one(ax, data, label, mode='plot'):
    if mode == 'twin' or mode == 'twinx':
      if '@' in label: mode = label.split('@')[-1]; label = '@'.join(label.split('@')[:-1])
      else: mode = 'plot';
      this_ax=ax.twinx(); this_ax._get_lines.get_next_color();# Handle TWIN
    elif mode=='twiny': #mode = 'plot';this_ax=ax.twiny(); next(this_ax._get_lines.prop_cycler);# Handle TWIN
      if '@' in label: mode = label.split('@')[-1]; label = '@'.join(label.split('@')[:-1])
      else: mode = 'plot';
      this_ax=ax.twiny(); this_ax._get_lines.get_next_color();# Handle TWIN
    elif mode=='twinxy':# mode = 'plot';this_ax=ax.twinx().twiny(); next(this_ax._get_lines.prop_cycler);# Handle TWIN
      if '@' in label: mode = label.split('@')[-1]; label = '@'.join(label.split('@')[:-1])
      else: mode = 'plot';
      this_ax=ax.twinx().twiny(); this_ax._get_lines.get_next_color();# Handle TWIN
    else: this_ax = ax
    if label == 'vline' or label == 'hline':
      for line in data: exec("this_ax.ax%s(line, ls='--', c='black')"%(label));
      return;
    elif label == 'annotate':  # annotate, format should be [{'title':, 'pos':}]
      for ana in data:
        title, pos, dist = ana['title'], ana['pos'], ana['dist'] if 'dist' in ana else (+15, -15)
        this_ax.annotate(title, xy=pos, xycoords='data', xytext=dist, textcoords='offset points', fontsize=12, arrowprops=dict(arrowstyle='-'), color='black')
      return
    if isinstance(data, pd.Series): x, y = data.index, data.values
    elif isinstance(data, dict) or isinstance(data, defaultdict): temp = sorted(data.items(), key=lambda x:x[0]);x, y = [x for x, y in temp], [y for x, y in temp]
    elif isinstance(data, list) or isinstance(data, np.ndarray): x, y = range(len(data)), data
    elif isinstance(data, tuple): x, y = data
    elif isinstance(data, pd.DataFrame): this_ax.table(cellText=data.round(3).values, colLabels=data.columns, rowLabels=data.index, loc='center', colWidths = [0.9/len(data.columns)]*len(data), label=label);this_ax.axis('off');return
    else: p('unknown data type', type(data)); sys.exit(1)
    if mode == 'plot':  this_ax.plot(x, y, label=label)
    elif mode == 'bar': this_ax.bar(range(len(x)), y, alpha=0.8, label=label); this_ax.set_xticklabels([x[int(i)] if i >= 0 and i < len(x) else 'NAN' for i in this_ax.get_xticks()]);
    #print([x[int(i)] if i >= 0 and i < len(x) else 'NAN' for i in this_ax.get_xticks()]); 
    elif mode == 'barh': this_ax.barh(range(len(x)), y, alpha=0.8, label=label); this_ax.set_yticklabels([x[int(i)] if i >= 0 and i < len(x) else 'NAN' for i in this_ax.get_yticks()]);
    elif mode == 'hist': this_ax.hist(data, bins=200, alpha=0.5, label=label)
    elif mode == 'scatter': this_ax.scatter(x, y, label=label, s=1)
    elif mode == 'pie': this_ax.pie(y, labels=x)
    elif mode == 'acf': plot_acf(pd.Series(y), this_ax, label=label);
    else: p('unknown mode', mode); sys.exit(1)

  @staticmethod
  def MultiPlot3(m, fig_name='default', show=False, prefix='', save_dir=os.environ['HOME']+'/MultiPlot', png_width=3, zoom=1.0, width_zoom=1., with_number = False, sc=1, full=False):
    # m = {subfig_title : {subfig_label: data -> list or dict}}
    s = time.time()
    #if not save_dir.startswith(os.environ['HOME']): save_dir = os.environ['HOME'] + '/' + save_dir
    os.system('install -d %s' % (save_dir))
    ksize = len(m.keys())
    fig_size = ((15 * zoom, 6 * zoom * width_zoom) if not full else (18 * zoom, 9 * zoom * width_zoom)) if png_width > 1 else (8 * zoom, 15 * zoom * width_zoom)
    ncol = min(math.ceil(math.sqrt(ksize)), png_width)
    nrow = min(math.ceil(ksize*1.0 / ncol), png_width) if png_width != 1 else ksize
    fig,ax = plt.subplots(nrows=nrow,ncols=ncol,figsize=fig_size);# fig.tight_layout()
    mitem = sorted(m.items(), key=lambda x: x[0])
    for count, (subfig_title, subfig_data) in enumerate(mitem):
      if count % (ncol*nrow) == 0 and count > 0: fig.tight_layout(); fig.savefig(save_dir+'/'+fig_name.replace('.', '_') + str(count) if with_number else fig_name.replace('.', '_')); fig,ax = plt.subplots(nrows=nrow,ncols=ncol,figsize=fig_size); print('finished @%d' %(count))  # open another fig
      this_ax = ax if nrow == 1 else ax[int(count/ncol)%nrow] if ncol == 1 else ax[int(count/ncol)%nrow, count%ncol];# this_ax.set_title(prefix+'@'+subfig_title); this_ax.grid()
      if isinstance(subfig_data, dict): # get into plot data
        #if isinstance(list(subfig_data.values())[0], int) or isinstance(list(subfig_data.values())[0], float):  # compatile for previous version
          #plt.plot(list(subfig_data.keys()), list(subfig_data.values()), label='empty');plt.grid();plt.legend()
          #continue
        for label, data in subfig_data.items():  # one label, one curve
          label = str(label)
          label, this_mode = ('@'.join(label.split('@')[:-1]), label.split('@')[-1]) if '@' in label else (label, 'plot')
          Plotor.plot_one(ax=this_ax, data=data, label=label, mode=this_mode)
          if this_mode != 'pie': this_ax.legend()
      else: Plotor.plot_one(ax=this_ax, data=subfig_data, label='empty', mode='plot'); this_ax.legend()
      this_ax.set_title(prefix+'@'+subfig_title); this_ax.grid()
      #if len(''.join(map(str, plt.xticks()[0]))) > 40:
      this_ax.xaxis.set_tick_params(rotation=30)
      #this_ax.tight_layout()
    fig.tight_layout();
    png_file = '%s/%s%s' % (save_dir, fig_name, '@%d'%(count) if with_number else '')
    fig.savefig(png_file.replace('.', '_'))
    if show: plt.show()
    else: RedPrint('Saving png to ', png_file, color='blue')
    #RedPrint('multiplot3 cost', time.time() - s)

if __name__ == '__main__':
  pt = Plotor()
  m = {"a":[i**2 for i in range(100)], "b":[i**3 for i in range(100)]}
  tm = {"a":{str(i):[(i+1)*j for j in range(100)] for i in range(2)}}
  tm['a']['test_twin'] = [i**2*10 for i in range(100)]
  m = {'IH': {"x":{1:398.26281569824187, 2:676.8502469238279}, 'vline':[1.5, 1.7], 'hxy_twin':[1,2,3], 'hline':[500], 'annotate':[{'title':'open', 'pos':(1.5, 400)}]}, 'IC': [543.9928998046876, 1008.4046122070322, 1472.891766406251, 1937.4634248046896], 'ni': [86.0, 142.0], 'IF': [430.87182810058596, 742.1164718994141]}
  #m = {str(i):[543.9928998046876, 1008.4046122070322, 1472.891766406251] for i in range(25)}
  #m = {'ni': {'raw': {datetime.date(2019, 3, 7): -50.0}, 'net': {datetime.date(2019, 3, 7): -74.0}}}
  a = {'20190912': {'netpnl': {'IC': [0, -0.8000000000001819, -0.6000000000003638, -3.600000000000364, -6.599999999999454]}, 'slip_twin': [292.1999999999998, 4.199999999999818, -10414.2, 3.4000000000005457, 10400.6, -15.0, -10447.2, 3.2000000000007276]}, 'huang':[1,2,3,4]}
  #a = {'20190912': {'netpnl': {start_ts +i*3600  :v for i, v in enumerate([0, -0.8000000000001819, -0.6000000000003638, -3.600000000000364, -6.599999999999454])}}}
  a = {'123':{'asd@scatter':{1:2, 2:3, 3:4}, 'vline':[1.2, 1.3]}, '2@bar':[1,2,3,4]}
  #pt.MultiPlot2(a, 'test', show=True)
  #pt.MultiPlot3({'123':([1,2,3], [1,2,3]), '1':{'1@scatter':(range(1000), range(100, 1100)),'2':(range(1000), range(1100, 2100)), 'annotate':[{'title':'asd', 'pos':(1,2), 'dist':(0,0)}]},'22':{'xhy':pd.Series({dt.datetime.strptime('20190101', '%Y%m%d'):2, dt.datetime.strptime('20200101', '%Y%m%d'):3})}, 'asd':{'asd@df':pd.DataFrame(np.reshape(range(100), (20,5)))}}, show=True)
  s = pd.Series({'a':3, 'b':2, 'c':4, 'd':8, 'ee': 10})
  #pt.MultiPlot3({'asd':{'a@bar':s, 'annotate':[{'pos':(i, ss/2),'dist':(0.05,0.05), 'title':'test'} for i, ss in enumerate(s)]}}, show=True)
  #pt.MultiPlot3({'asd':{'a@bar':s, 'b@bar@twin':[10, 20, 25]}}, show=True, full=True)
  pt.MultiPlot3({'asd':{'a@acf':list(range(100))}}, show=True, full=True)
  pt.MultiPlot3({'asd':{'a@pie':{'a':1, 'b':2, 'c':3}}}, show=True, full=True)



type_list = ['Uninited','Acc','Rej','Cancelled','CancelRej',
'Filled','Pfilled','Position','Unknown']
side_int = {"BUY":1, 'SELL':2}
int_side = {-1:'UNKNOWN', 1:"BUY", 2:'SELL'}
type_int = {type_list[i]:i for i in range(len(type_list))}
int_type = {i:type_list[i] for i in range(len(type_list))}
int_type[-1] = 'uninit'

class ExchangeInfo:
  def __init__(self):
    self.show_time = 1000000000.0
    self.shot_time = 1000000000.0
    self.type = -1;
    self.ticker = 'none';
    self.order_ref = 'none';
    self.trade_size = -1;
    self.trade_price = -1.0;
    self.side = -1;
    self.reason = 'none';

  @property
  def fmt(): return "4Qi32s32sidi64s4s"

  def Check(self):
    return self.type == 5

  def __str__(self):
    show_str = ''
    split_c = ' '
    show_str += repr(self.show_time)
    show_str += split_c
    show_str += repr(self.shot_time)
    show_str += split_c
    show_str += 'exchangeinfo' + split_c
    show_str += self.order_ref + split_c
    show_str += '|' + split_c
    show_str += str(self.trade_price) + '@'+ str(self.trade_size) + split_c
    show_str += str(int_type[self.type]) + split_c
    show_str += self.ticker+split_c
    show_str += int_side[self.side]
    show_str += split_c
    show_str += self.reason
    return show_str

if __name__ == '__main__':
  with open('/today/filled') as f:
    for l in f:
      ei = ExchangeInfo()
      ei.construct(l)
      ei.Show()



import time
import pandas as pd
import struct

DEPTH = 5
class BookTicker:
  def __init__(self):
    self.tk_= "none"
    self.ts_ = 1000000000
    self.uts_ = 1000
    self.bp0_ = self.ap0_ = self.bs0_ = self.as0_ = 0.
    self.ex_ = 'B'
    self.type_ = 'S'
  
  @property
  def fmt(self): return '8s16s2Q4dcc6s'

  def pack(self):
    return struct.pack(self.fmt, bytes(self.tk_, 'utf-8'), bytes(self.tk_, 'utf-8'), self.ts_, self.uts_, self.bp0_, self.ap0_, self.bs0_, self.as0_, self.ex_, self.type_,  b'')

  def __str__(self):
    split_char = ' '
    show_content = ""
    show_content += str(self.ts_) + '.' + str(self.uts_)
    show_content += split_char
    show_content += "BOOKTICKER"
    show_content += split_char
    show_content += self.tk_
    show_content += split_char
    show_content += '|'
    show_content += split_char
    show_content += str(self.bp0_) + split_char + str(self.ap0_) + ' | ' + str(self.bs0_) + ' x ' + str(self.as0_) + split_char
    show_content += str(self.ex_)
    show_content += split_char
    show_content += str(self.type_)
    return show_content

if __name__ == '__main__':
  shot = BookTicker()
  shot.tk = "HAHA"
  m = {}
  for i, j in zip(shot.cols, shot.to_csv().split(',')): m[i] = j
  print(pd.Series(m))



import sys
import libpyutil as pyutil
import libpytrade as pymmm
from ..util.util import *
from .struct_util import *
import os
import pandas as pd
import gzip
import time
import glob

class Reader:
  def __init__(self):
    pass

  # load 10G data cost 40s, gzip.read() cost 10s, open.read()(not zipped file) 1s
  @staticmethod
  def load_shot_file(s, file_path):
    shot_size = get_size(s)
    shot_content = gzip.open(file_path).read() if file_path.endswith('.gz') else open(file_path, 'rb').read()
    if len(shot_content) % shot_size != 0: print("%s filesize not shot's times" %(file_path))
    self.shot_size = int(len(shot_content) / shot_size)
    for i in range(shot_size): yield unpack(s, shot_content[i*shot_size:(i+1)*shot_size])

  @staticmethod
  def load_shot_files(files): return [Reader.load_shot_file(f) for f in files]

  @staticmethod
  def load_bt(tickers={'BTC_USDT'}, start_date='20200101', end_date='$TODAY', mp=False, cols= ['bp0_', 'ap0_', 'bs0_', 'as0_', 'ex_', 'type_']):
    start = time.time()
    fd = pymmm.Find(start_date, end_date, tickers)
    tfm = defaultdict(list)
    for date, f in fd.items():
      for topic, path in f.items(): tfm[topic].append(path)
    if mp: df = pd.concat(MPRun(Reader.bt_load,  [[f, topic, cols] for topic, fm in tfm.items() for f in fm]))
    else: df = pd.concat([Reader.bt_load(f, topic, cols) for topic, fm in tfm.items() for f in fm])
    df['sec'] = df['ts_'] % (24*3600)
    df = df[((df['sec'] >= 3600) & (df['sec'] <= 7*3600)) | ((df['sec'] >= 13*3600) & (df['sec'] <= 18.5 * 3600))]
    del df['ts_'], df['sec']
    RedPrint('load cost', time.time()-start,  color='green')
    return df.set_index('dt')

  @staticmethod
  def bt_load(file_path, topic, cols=['bp0_', 'ap0_', 'bs0_', 'as0_', 'ex_', 'type_']):
    shot = BookTicker()
    m = 'a8,a16,i8,i8,' + 'f8,' * 4 + 'a1,a1,a6'
    df = pd.DataFrame(np.frombuffer(open('%s'%(file_path), 'rb').read()[40:], dtype=m))
    df.columns = ['vptr'] + list(shot.__dict__.keys()) + ['left']
    del df['vptr'], df['left']
    tme = df['ts_'] + df['uts_'] / 1e6
    del df['uts_']
    df['tk_'] = df['tk_'].apply(lambda x: str(x)[2:].split('\\x0')[0].split("'")[0])
    df['topic'] = topic
    dtt = pd.to_datetime(tme, unit='s', utc=True).dt.tz_convert('Asia/Shanghai')
    df['dt'] = dtt
    if 'min' in topic or 'daily' in topic:
      rdf = df[set(['ts_', 'tk_', 'dt', 'topic'] + cols + ['bp1_', 'bp2_', 'ap1_', 'ap2_'])]
      rdf = rdf[~np.isnan(rdf['bp2_'])]
      del_cols = ['lp_', 'ls_']
      for col in del_cols: 
        if col in rdf.columns: del rdf[col]
      return rdf.rename(columns={'bp1_':'open', 'bp2_':'low', 'ap1_':'close', 'ap2_':'high'})
    cols = list(set(['ts_', 'tk_', 'dt', 'topic'] + cols))
    return df[cols]


  @staticmethod
  def load(tickers={'JM1'}, start_date='20200101', end_date='$TODAY', mp=False, cols= ['bp0_', 'ap0_', 'bs0_', 'as0_', 'lp_', 'tvr_', 'ls_', 'vol_', 'oi_']):
    start = time.time()
    fd = pymmm.Find(start_date, end_date, tickers)
    tfm = defaultdict(list)
    for date, f in fd.items():
      for topic, path in f.items(): tfm[topic].append(path)
    if mp: df = pd.concat(MPRun(Reader.load_shot,  [[f, topic, cols] for topic, fm in tfm.items() for f in fm]))
    else: df = pd.concat([Reader.load_shot(f, topic, cols) for topic, fm in tfm.items() for f in fm])
    df['sec'] = df['ts_'] % (24*3600)
    df = df[((df['sec'] >= 3600) & (df['sec'] <= 7*3600)) | ((df['sec'] >= 13*3600) & (df['sec'] <= 18.5 * 3600))]
    del df['ts_'], df['sec']
    RedPrint('load cost', time.time()-start,  color='green')
    return df.set_index('dt')

  @staticmethod
  def load_shot(file_path, topic, cols=['bp0_', 'ap0_', 'bs0_', 'as0_', 'lp_', 'tvr_', 'ls_', 'vol_', 'oi_']):
    shot = MarketSnapshot()
    m = 'a8,a16,i8,i8,' + 'f8,' * 10 + 'i4,' * 10 + 'f8,f8,' + 'i4,' * 3 + 'i4'
    df = pd.DataFrame(np.frombuffer(open('%s'%(file_path), 'rb').read()[40:], dtype=m))
    #del df['f24'], df['f29']
    del df['f28']
    df.columns = ['vptr'] + list(shot.__dict__.keys())
    del df['vptr']
    tme = df['ts_'] + df['uts_'] / 1e6
    del df['uts_']
    df['tk_'] = df['tk_'].apply(lambda x: str(x)[2:].split('\\x0')[0].split("'")[0])
    df['topic'] = topic
    dtt = pd.to_datetime(tme, unit='s', utc=True).dt.tz_convert('Asia/Shanghai')
    #df['date'] = dtt.dt.date#.apply(lambda x: int(x.strftime('%Y%M%d')))
    #df['time'] = dtt.dt.time
    df['dt'] = dtt
    if 'min' in topic or 'daily' in topic:
      rdf = df[set(['ts_', 'tk_', 'dt', 'topic'] + cols + ['bp1_', 'bp2_', 'ap1_', 'ap2_'])]
      rdf = rdf[~np.isnan(rdf['bp2_'])]
      del_cols = ['lp_', 'ls_']
      for col in del_cols: 
        if col in rdf.columns: del rdf[col]
      return rdf.rename(columns={'bp1_':'open', 'bp2_':'low', 'ap1_':'close', 'ap2_':'high'})
    #return df[['tk_', 'date', 'time', 'topic'] + cols]
    cols = list(set(['ts_', 'tk_', 'dt', 'topic'] + cols))
    return df[cols]

if __name__=='__main__':
  df = Reader.load_tick(['IC1', 'NI1'], start_date=20200928, end_date=20200928)
  print(df)
